
// layer.h

#if ! defined clayer_h
#define clayer_h


//#include "prdfclr.h"
#include "assert.h"
#include "glblcnst.h"
#include "DocColorModel.h"
#include "refcountptr.h"
#include "scaler.h"
#include "imagedata.h"
#include <vector>
#include "serialize.h"


class TDocument;
class ImageData;


enum
{
	Color1 = 1,
	Color2 = 2
};


enum
{
	ClassID_Layer1Bit = 1,
	ClassID_LayerContone = 2,
	ClassID_LayerContoneQRes = 3
};


// Layer represents a shade within a document.
class
	Layer:
public
	Serializable_Group
{
private:
	//long			fserial;
	bool			hidden;
	bool			selected;
	bool			sourceLayer;
protected:
	MultiInkColor			m_Color;
	MultiInkColor			m_Color2;
	Serializable_long		m_Width;
	Serializable_long		m_Height;
	//std::wstring			m_Name;
	Serializable_wstring	m_Name;
	gd::ipoint*				selectionOffset;
	Serializable_long		textureId;
	
	long					layerID;

	static long				NewID();
public:
	//gd::irect		fold_active_rect;
	//gd::irect		factive_rect;

	Layer( const std::wstring& name, long width, long height );
	virtual ~Layer() {}

	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray;

		m_Color.SetFieldID( 1 );
		childrenArray.push_back( &m_Color );

		m_Color2.SetFieldID( 2 );
		childrenArray.push_back( &m_Color2 );

		m_Width.SetFieldID( 3 );
		childrenArray.push_back( &m_Width );

		m_Height.SetFieldID( 4 );
		childrenArray.push_back( &m_Height );

		m_Name.SetFieldID( 7 );
		childrenArray.push_back( &m_Name );

		return childrenArray;
	}

	long LayerID() { return layerID; }

	//long the_serial( void ) { return fserial; }
	bool get_hidden( void ) { return hidden; }

	void SetSelected( bool selectedParam ) { selected = selectedParam; }
	bool Selected() { return selected; }
	void SetSourceLayer( bool sourceLayerparam ) { sourceLayer = sourceLayerparam; }
	bool SourceLayer() { return sourceLayer; }

	virtual void RenderOnPixmaps( const std::vector< RefCountPtr< gd::Pixmap_8Bit > >& multiPixmap, Scaler scaler ) = 0;
	virtual void StartUndoableAction();

	long NumberOfInks() const
	{
		return m_Color.NumberOfInks();
	}
	//InkShade Ink( long index )
	//{
	//	return m_Color.Ink( index );
	//}

	//void AddInkValue( InkSpecPtr inkSpec, long shade );
	//bool			SourceLayer() { return m_SourceLayer; }
	MultiInkColor*	InkColor();
	MultiInkColor*	InkColor2();

	void			SetInkColor( const MultiInkColor& color, const MultiInkColor& color2 );

	RGBColor		GetRGBColor();
	std::wstring	Name();
	void			SetName( const std::wstring& name ) { m_Name.value = name; }
	void			SetInks( const MultiInk& inksParam );
	long			InkShade255( long index, long whichColor );
	long			InkShade100( long index, long whichColor );
	void			SetShade100( long inkIndex, long shadeValue, long whichColor );
	bool			InkIsProcessColor( long index, long whichColor );
	RGBColor		InkRGBColor( int index, int whichColor );
	std::wstring	InkName( int index, int whichColor );
	virtual bool	IsContone() { return false; }
	virtual	RefCountPtr< gd::Pixmap_8Bit >
					Render1InkOn1BytePixmap( long inkIndex, const gd::irect& pixmapBounds, Scaler scaler )
					{
						assert( false );
						return RefCountPtr< gd::Pixmap_8Bit >( 0 );
					}
	virtual RefCountPtr< gd::Pixmap_8Bit >
					RenderOn1BytePixmap( const gd::irect& pixmapBounds, Scaler scaler )
					{
						assert( false );
						return RefCountPtr< gd::Pixmap_8Bit >( 0 );
					}
	virtual RefCountPtr< gd::Pixmap_1Bit >
					RenderOn1BitPixmap( const gd::irect& pixmapBounds, Scaler scaler )
					{
						assert( false );
						return RefCountPtr< gd::Pixmap_1Bit >( 0 );
					}
	virtual gd::BWPixmapPtr Get1BitImage( const gd::irect& getRect )
	{
		assert( false );
		return gd::BWPixmapPtr( 0 );
	}
	virtual void AddImage( const gd::Pixmap_1Bit& imagePixmap ) { assert( false ); }
	virtual void EraseImage( const gd::Pixmap_1Bit& imagePixmap ) { assert( false ); }
	virtual void ApplyImage( const gd::Pixmap_8Bit& imagePixmap ) { assert( false ); }
	virtual void ApplyBlend( const gd::Pixmap_8Bit& blendPixmap, const gd::Pixmap_1Bit& brushMask, bool antiAliasBrush = true ) { assert( false ); }
	virtual bool PointIsPainted( gd::ipoint mouseLocation ) { return false; }
	virtual void SetCurrentColorFromPixel( const gd::ipoint& mouseLocation ) {}
	//virtual Layer* ExtractLayerPortion( const gd::irect& selectRect ) = 0;
	virtual bool Matches( Layer& otherLayer ) = 0;
	void SetColorBlack();

	long ProcessCyanShade();
	long ProcessMagentaShade();
	long ProcessYellowShade();
	long ProcessBlackShade();

	void SetProcessCyanShade( long shade );
	void SetProcessMagentaShade( long shade );
	void SetProcessYellowShade( long shade );
	void SetProcessBlackShade( long shade );

	virtual void CopyPixmapToImage( gd::Pixmap_1Bit const & pixMap ) { assert( false ); }
	virtual void CopyImageToPixmap( gd::Pixmap_1Bit const & pixMap ) { assert( false ); }

	long			Width() { return m_Width.value; }
	long			Height() { return m_Height.value; }
	virtual void	CreateSelection( const gd::irect& selectRect, gd::ipoint* selectionOffsetParam ) = 0;
	virtual void	AddSelectionToLayer( const gd::ipoint& selectionOffset ) = 0;
	virtual void	DeleteSelection() = 0;
	virtual void	UndoLastAction() = 0;
	virtual bool	NeedsSaving() = 0;
	virtual void	SetHasBeenSaved() = 0;

	void			UpdateInks( MultiInk newInks );
	virtual void	SaveDataToFile( const gd::GDFile& saveFile ) = 0;
	virtual void	InitializeUndoGrids() { assert( false ); }
	
	void			SetTexture( long textureId_ ) { textureId = textureId_; }
	long			Texture() { return textureId; }

	virtual char*	DataPtr() = 0;
	virtual long	DataSize() = 0;
	virtual void	CopyFromCmykData( unsigned char* sourceData, long sourceDataSize, long sourceWidth, long sourceHeight, long sourceRowbytes ) {}
};


class
	Layer1Bit:
public
	Layer
{
private:
	SerializablePointer< ImageDataPtr > image;
	//ImageDataPtr image;
	ImageDataPtr selectionImage;
public:
	Layer1Bit( const std::wstring& name, long width, long height );
	Layer1Bit( const std::wstring& name, gd::BWPixmapPtr& layerPixmap );

	~Layer1Bit();

	void			RenderOnPixmaps( const std::vector< RefCountPtr< gd::Pixmap_8Bit > >& multiPixmap, Scaler scaler );
	RefCountPtr< gd::Pixmap_8Bit >
					RenderOn1BytePixmap( const gd::irect& pixmapBounds, Scaler scaler );
	RefCountPtr< gd::Pixmap_1Bit >
					RenderOn1BitPixmap( const gd::irect& pixmapBounds, Scaler scaler );

	void			CopyPixmapToImage( gd::Pixmap_1Bit const & pixMap );
	void			CopyImageToPixmap( gd::Pixmap_1Bit const & pixMap );
	bool			NeedsSaving();
	void			SetHasBeenSaved();
	void			UndoLastAction();
	void			StartUndoableAction();
	gd::BWPixmapPtr		Get1BitImage( const gd::irect& getRect );
	void			AddImage( const gd::Pixmap_1Bit& imagePixmap );
	void			EraseImage( const gd::Pixmap_1Bit& imagePixmap );
	bool			PointIsPainted( gd::ipoint mouseLocation );
	Layer*			ExtractLayerPortion( const gd::irect& selectRect );
	bool			Matches( Layer& otherLayer );
	void			CreateSelection( const gd::irect& selectRect, gd::ipoint* selectionOffsetParam );
	void			AddSelectionToLayer( const gd::ipoint& selectionOffset );
	void			DeleteSelection();
	void			SaveDataToFile( const gd::GDFile& saveFile );
	long			ClassHeirarchyMemberID() { return ClassID_Layer1Bit; }

	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray = Layer::SerializableChildren();

		image.SetFieldID( 5 );
		childrenArray.push_back( &image );

		return childrenArray;
	}
	void InitializeUndoGrids();
	char*	DataPtr()
	{
		return image.value->DataPtr();
	}
	long	DataSize()
	{
		return image.value->DataSize();
	}
};


class
	LayerContone:
public
	Layer
{
private:
	std::vector< ImageDataPtr > images;
public:
	LayerContone( const std::wstring& name, long width, long height, long numberOfInks );
	~LayerContone();

	static long		ClassFieldID() { return 2; }

	void			RenderOnPixmaps( const std::vector< RefCountPtr< gd::Pixmap_8Bit > >& multiPixmap, Scaler scaler );

	//void			CopyPixmapToImage( gd::Pixmap_1Bit const & pixMap );
	//void			CopyImageToPixmap( gd::Pixmap_1Bit const & pixMap );
	bool			NeedsSaving();
	void			SetHasBeenSaved();
	void			UndoLastAction();
	void			StartUndoableAction();
	bool			IsContone() { return true; }
	RefCountPtr< gd::Pixmap_8Bit >
					Render1InkOn1BytePixmap( long inkIndex, const gd::irect& pixmapBounds, Scaler scaler );
	gd::BWPixmapPtr		Get1BitImage( const gd::irect& getRect );
	void			AddImage( const gd::Pixmap_1Bit& imagePixmap );
	void			EraseImage( const gd::Pixmap_1Bit& imagePixmap );
	void			InitializeUndoGrids();
	long			ClassHeirarchyMemberID() { return ClassID_LayerContone; }
	char*	DataPtr()
	{
		assert( false );
		return 0;
		//return image.value->DataPtr();
	}
	long	DataSize()
	{
		assert( false );
		return 0;
		//return image.value->DataSize();
	}
};


class
	LayerContoneQRes:
public
	Layer
{
private:
	SerializableVectorPointers< ImageDataPtr > images;
	//std::vector< ImageDataPtr > images;
	std::vector< ImageDataPtr > selectionImages;
public:
	LayerContoneQRes( const std::wstring& name, long width, long height, long numberOfInks );
	~LayerContoneQRes();

	static long		ClassFieldID() { return 3; }

	void			RenderOnPixmaps( const std::vector< RefCountPtr< gd::Pixmap_8Bit > >& multiPixmap, Scaler scaler );

	//void			CopyPixmapToImage( gd::Pixmap_1Bit const & pixMap );
	//void			CopyImageToPixmap( gd::Pixmap_1Bit const & pixMap );
	bool			NeedsSaving();
	void			SetHasBeenSaved();
	void			UndoLastAction();
	void			StartUndoableAction();
	bool			IsContone() { return true; }
	RefCountPtr< gd::Pixmap_8Bit >
					Render1InkOn1BytePixmap( long inkIndex, const gd::irect& pixmapBounds, Scaler scaler );
	gd::BWPixmapPtr		Get1BitImage( const gd::irect& getRect );
	void			AddImage( const gd::Pixmap_1Bit& imagePixmap );
	void			EraseImage( const gd::Pixmap_1Bit& imagePixmap );
	void			ApplyImage( const gd::Pixmap_8Bit& imagePixmap );
	gd::GrayPixmapPtr   CreateQResGrayPixmap( const gd::Pixmap_1Bit& imagePixmap, bool antiAliasBrush );
	void			ApplyBlend( const gd::Pixmap_8Bit& blendPixmap, const gd::Pixmap_1Bit& brushMask, bool antiAliasBrush = true );
	bool			PointIsPainted( gd::ipoint mouseLocation );
	void			SetCurrentColorFromPixel( const gd::ipoint& mouseLocation );
	Layer*			ExtractLayerPortion( const gd::irect& selectRect );
	bool			Matches( Layer& otherLayer );
	void			CreateSelection( const gd::irect& selectRect, gd::ipoint* selectionOffsetParam );
	void			AddSelectionToLayer( const gd::ipoint& selectionOffset );
	void			DeleteSelection();
	void			SaveDataToFile( const gd::GDFile& saveFile );
	void			InitializeUndoGrids();
	long			ClassHeirarchyMemberID() { return ClassID_LayerContoneQRes; }
	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray = Layer::SerializableChildren();

		images.SetFieldID( 6 );
		childrenArray.push_back( &images );

		return childrenArray;
	}
	char*	DataPtr()
	{
		assert( false );
		return 0;
		//return image.value->DataPtr();
	}
	long	DataSize()
	{
		assert( false );
		return 0;
		//return image.value->DataSize();
	}
	void			CopyFromCmykData( unsigned char* sourceData, long sourceDataSize, long sourceWidth, long sourceHeight, long sourceRowbytes );
};


typedef RefCountPtr< Layer > LayerPtr;


#endif

